=== modified file 'NEWS'
--- NEWS	2007-09-16 19:29:00 +0000
+++ NEWS	2007-09-17 05:33:56 +0000
@@ -39,6 +39,9 @@
      longer throws away the profiling data if this command is run with
      ``--lsprof-file callgrind.out.ci`` say. (Ian Clatworthy)
 
+   * ``WorkingTree.rename_one`` will now raise an error if normalisation of the
+     new path causes bzr to be unable to access the file. (Robert Collins)
+
   API BREAKS:
 
    * The ``VersionedFile`` interface now allows content checks to be bypassed

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2007-09-14 02:25:32 +0000
+++ bzrlib/dirstate.py	2007-09-17 05:33:56 +0000
@@ -365,7 +365,7 @@
         # find the location in the block.
         # check its not there
         # add it.
-        #------- copied from inventory.make_entry
+        #------- copied from inventory.ensure_normalized_name - keep synced.
         # --- normalized_filename wants a unicode basename only, so get one.
         dirname, basename = osutils.split(path)
         # we dont import normalized_filename directly because we want to be

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2007-09-04 03:53:07 +0000
+++ bzrlib/inventory.py	2007-09-17 05:33:56 +0000
@@ -1371,6 +1371,7 @@
         This does not move the working file.
         """
         file_id = osutils.safe_file_id(file_id)
+        new_name = ensure_normalized_name(new_name)
         if not is_valid_name(new_name):
             raise BzrError("not an acceptable filename: %r" % new_name)
 
@@ -1418,7 +1419,21 @@
         file_id = generate_ids.gen_file_id(name)
     else:
         file_id = osutils.safe_file_id(file_id)
-
+    name = ensure_normalized_name(name)
+    try:
+        factory = entry_factory[kind]
+    except KeyError:
+        raise BzrError("unknown kind %r" % kind)
+    return factory(file_id, name, parent_id)
+
+
+def ensure_normalized_name(name):
+    """Normalize name.
+
+    :raises InvalidNormalization: When name is not normalized, and cannot be
+        accessed on this platform by the normalized path.
+    :return: The NFC/NFKC normalised version of name.
+    """
     #------- This has been copied to bzrlib.dirstate.DirState.add, please
     # keep them synchronised.
     # we dont import normalized_filename directly because we want to be
@@ -1426,17 +1441,12 @@
     norm_name, can_access = osutils.normalized_filename(name)
     if norm_name != name:
         if can_access:
-            name = norm_name
+            return norm_name
         else:
             # TODO: jam 20060701 This would probably be more useful
             #       if the error was raised with the full path
             raise errors.InvalidNormalization(name)
-
-    try:
-        factory = entry_factory[kind]
-    except KeyError:
-        raise BzrError("unknown kind %r" % kind)
-    return factory(file_id, name, parent_id)
+    return name
 
 
 _NAME_RE = None

=== modified file 'bzrlib/tests/test_workingtree_4.py'
--- bzrlib/tests/test_workingtree_4.py	2007-04-26 22:56:01 +0000
+++ bzrlib/tests/test_workingtree_4.py	2007-09-17 05:33:56 +0000
@@ -419,12 +419,12 @@
         tree.add(['b'], ['b-id'])
 
         try:
-            tree.rename_one('a', u'b\xb5rry')
-            new_path = u'b\xb5rry'
+            new_path = u'b\u03bcrry'
+            tree.rename_one('a', new_path)
         except UnicodeEncodeError:
             # support running the test on non-unicode platforms
-            tree.rename_one('a', 'c')
             new_path = 'c'
+            tree.rename_one('a', new_path)
         self.assertEqual(new_path, tree.id2path('a-id'))
         tree.commit(u'b\xb5rry')
         tree.unversion(['a-id'])

=== modified file 'bzrlib/tests/workingtree_implementations/test_rename_one.py'
--- bzrlib/tests/workingtree_implementations/test_rename_one.py	2007-02-26 16:19:02 +0000
+++ bzrlib/tests/workingtree_implementations/test_rename_one.py	2007-09-17 05:33:56 +0000
@@ -308,3 +308,10 @@
                                ('c', 'c-id')], tree)
         self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
+
+    def test_rename_to_denormalised_fails(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'])
+        self.assertRaises((errors.InvalidNormalization, UnicodeEncodeError),
+            tree.rename_one, 'a', u'b\xb5rry')

